==> evlan-0.3.3.tbz2: evlan-0.3.3/iochat-0.1/IoChat.java <==
/* Io Chat - Chat applet for Io Community Manager.
 * Copyright (C) 2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.net.URL;
import java.util.Vector;

/* You know what would be wonderful?  If browsers which claim to
 * support LiveConnect actually all did so *correctly*.  A short list
 * of problems I have had to work around in this applet:
 *
 * - Although LiveConnect is supposed to convert Java objects of type
 *   String, Integer, Boolean, etc. into the appropriate JavaScript
 *   primitives when passed to JavaScript, many implementations prefer
 *   instead to simply wrap the java object as if it were a generic
 *   Object.  Thus if you pass a String to JavaScript, you find that
 *   you must call .length() as a method rather than query .length as
 *   a field to get its length, etc.  This can be solved in JavaScipt
 *   by concatenating the string with an empty string, after which
 *   point it will be seen as a primitive string.  A bigger problem is
 *   that Booleans will remain objects and thus using them in
 *   conditionals will act as if they are always true (because you are
 *   actually checking if the value is null, as opposed to checking if
 *   it is false).  Since only *some* implementations have this
 *   problem and not others, I have resorted to passing string values
 *   "true" and "false" rather than booleans.
 * - I have yet to find a browser which actually lets you pass in
 *   objects from JavaScript and manipulate them as JSObjects.
 *   Although this seems like core functionality of LiveConnect, it
 *   seems that what I get is a JSObject for which the call() method
 *   simply refuses to function as documented.  Fortunately, most
 *   browsers implement JSObject.getWindow(), returning a JSObject
 *   which *does* work.
 * - JSObject.getWindow() throws an exception with no description in
 *   Safari.  Thanks, Safari.  Also, JavaScript objects passed in as
 *   method parameters are always null.  Thus, there is apparently no
 *   way at all to communicate from Java to JavaScript in Safari.  To
 *   work around this, I added a polling mode, where JavaScript must
 *   poll the applet for the latest messages every half-second or so.
 * - Windows users who haven't installed Java manually will typically
 *   find themselves running Java 1.1.  Java 1.1 is like the fucking
 *   dark ages.  There is no ArrayList, there is no HashMap, there is
 *   no UTF-8.  There is, however, Vector, Hashtable, and the ISO-
 *   whatever-1 character set.  This applet is designed to work
 *   properly in Java 1.1, and also attempts to use UTF-8 where
 *   available. */
public class IoChat extends java.applet.Applet implements Runnable
{
   private CommandStream mConnection;
   private netscape.javascript.JSObject mWindow;
   private java.applet.AudioClip mBeepClip;

   private Vector mCommands;

   public void stop()
   {
      destroy();
   }

   public void destroy()
   {
      try
      {
         if(mConnection != null)
            mConnection.close();
      }
      catch(java.io.IOException e)
      {
         e.printStackTrace();
      }
   }

   //Called from JavaScript to initiate connection.
   public String connect(String cookies)
   {
      try
      {
         mWindow = netscape.javascript.JSObject.getWindow(this);
      }
      catch(Exception e)
      {
         //Safari throws from getWindow()...
         mWindow = null;
      }

      if(mWindow == null)
         mCommands = new Vector();  //needed for polling mode

      try
      {
         URL documentBase = getDocumentBase();

         String beepLocation = getParameter("beep-clip");
         if(beepLocation != null)
            mBeepClip = getAudioClip(documentBase, beepLocation);

         String channelPath = getParameter("channel");
         URL channel;
         if(channelPath == null)
            channel = getDocumentBase();
         else
            channel = new URL(documentBase, channelPath);

         String host = channel.getHost();
         mConnection = new CommandStream(new java.net.Socket(host, 80));

         String request =
            "GET " + channel.getFile() + " HTTP/1.1\r\n" +
            "Host: " + host + "\r\n";
         if(cookies.length() > 0)
            request += "Cookie: " + cookies + "\r\n";
         request += "Upgrade: IoChat/0.1.0\r\n\r\n";
         mConnection.writeData(request.getBytes("ISO-8859-1"));

         String line = mConnection.read();
         if(!checkResponseLine(line))
            throw new java.io.IOException("Expected \"HTTP/1.1 101 Switching Protocols\", got \"" + line + "\".");

         while(!line.equals("") && !line.equals("\r"))
            line = mConnection.read();

         (new Thread(this)).start();
      }
      catch(java.io.IOException e)
      {
         chatError(e.getMessage());
         chatError("Failed to connect.");
         e.printStackTrace();
      }

      return (mWindow == null)?"true":"false";
   }

   public void submit(String text)
   {
      try {mConnection.write(text);} catch(java.io.IOException e) {}
   }

   public void beep()
   {
      if(mBeepClip != null)
         mBeepClip.play();
   }

   public void run()
   {
      try
      {
         while(true)
         {
            String line = mConnection.read();
            if(line == null)
               throw new java.io.IOException("Connection terminated.");
            chatCommand(new Command(line));
         }
      }
      catch(Throwable e)
      {
         chatError(e.getMessage());
         chatError("Lost connection to server.");
         e.printStackTrace();
      }
   }

   private void chatCommand(Command command)
   {
      if(mWindow == null)
      {
         synchronized(this) {mCommands.addElement(command);}
      }
      else
      {
         Object[] args = {command.name, command.parameters, (command.text != null)?"true":"false", command.text};
         mWindow.call("chatCommand", args);
      }
   }

   private void chatError(String message)
   {
      chatCommand(new Command("error", message));
   }

   public Object[][] pollCommands()
   {
      Object[][] result;

      synchronized(this)
      {
         int count = mCommands.size();
         result = new Object[count][];

         for(int i = 0; i < count; i++)
         {
            Command command = (Command)mCommands.elementAt(i);
            result[i] = new Object[4];
            result[i][0] = command.name;
            result[i][1] = command.parameters;
            result[i][2] = (command.text != null)?"true":"false";
            result[i][3] = command.text;
         }

         mCommands.removeAllElements();
      }

      return result;
   }

   private boolean checkResponseLine(String line)
   {
      if(!line.startsWith("HTTP/1."))
         return false;

      int pos = line.indexOf(' ');
      if(pos < 0)
         return false;

      if(!line.startsWith("101 ", pos+1))
         return false;
      return true;
   }
}

==> evlan-0.3.4.tbz2: evlan-0.3.4/iochat-0.1/IoChat.java <==
/* Io Chat - Chat applet for Io Community Manager.
 * Copyright (C) 2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.net.URL;
import java.util.Vector;

/* You know what would be wonderful?  If browsers which claim to
 * support LiveConnect actually all did so *correctly*.  A short list
 * of problems I have had to work around in this applet:
 *
 * - Although LiveConnect is supposed to convert Java objects of type
 *   String, Integer, Boolean, etc. into the appropriate JavaScript
 *   primitives when passed to JavaScript, many implementations prefer
 *   instead to simply wrap the java object as if it were a generic
 *   Object.  Thus if you pass a String to JavaScript, you find that
 *   you must call .length() as a method rather than query .length as
 *   a field to get its length, etc.  This can be solved in JavaScipt
 *   by concatenating the string with an empty string, after which
 *   point it will be seen as a primitive string.  A bigger problem is
 *   that Booleans will remain objects and thus using them in
 *   conditionals will act as if they are always true (because you are
 *   actually checking if the value is null, as opposed to checking if
 *   it is false).  Since only *some* implementations have this
 *   problem and not others, I have resorted to passing string values
 *   "true" and "false" rather than booleans.
 * - I have yet to find a browser which actually lets you pass in
 *   objects from JavaScript and manipulate them as JSObjects.
 *   Although this seems like core functionality of LiveConnect, it
 *   seems that what I get is a JSObject for which the call() method
 *   simply refuses to function as documented.  Fortunately, most
 *   browsers implement JSObject.getWindow(), returning a JSObject
 *   which *does* work.
 * - JSObject.getWindow() throws an exception with no description in
 *   Safari.  Thanks, Safari.  Also, JavaScript objects passed in as
 *   method parameters are always null.  Thus, there is apparently no
 *   way at all to communicate from Java to JavaScript in Safari.  To
 *   work around this, I added a polling mode, where JavaScript must
 *   poll the applet for the latest messages every half-second or so.
 * - Windows users who haven't installed Java manually will typically
 *   find themselves running Java 1.1.  Java 1.1 is like the fucking
 *   dark ages.  There is no ArrayList, there is no HashMap, there is
 *   no UTF-8.  There is, however, Vector, Hashtable, and the ISO-
 *   whatever-1 character set.  This applet is designed to work
 *   properly in Java 1.1, and also attempts to use UTF-8 where
 *   available. */
public class IoChat extends java.applet.Applet implements Runnable
{
   private CommandStream mConnection;
   private netscape.javascript.JSObject mWindow;
   private java.applet.AudioClip mBeepClip;

   private Vector mCommands;

   public void stop()
   {
      destroy();
   }

   public void destroy()
   {
      try
      {
         if(mConnection != null)
            mConnection.close();
      }
      catch(java.io.IOException e)
      {
         e.printStackTrace();
      }
   }

   //Called from JavaScript to initiate connection.
   public String connect(String cookies)
   {
      try
      {
         mWindow = netscape.javascript.JSObject.getWindow(this);
      }
      catch(Exception e)
      {
         //Safari throws from getWindow()...
         mWindow = null;
      }

      if(mWindow == null)
         mCommands = new Vector();  //needed for polling mode

      try
      {
         URL documentBase = getDocumentBase();

         String beepLocation = getParameter("beep-clip");
         if(beepLocation != null)
            mBeepClip = getAudioClip(documentBase, beepLocation);

         String channelPath = getParameter("channel");
         URL channel;
         if(channelPath == null)
            channel = getDocumentBase();
         else
            channel = new URL(documentBase, channelPath);

         String host = channel.getHost();
         int port = channel.getPort();
         mConnection = new CommandStream(new java.net.Socket(host, port));

         if(port != 80)
            host = host + ":" + port;

         String request =
            "GET " + channel.getFile() + " HTTP/1.1\r\n" +
            "Host: " + host + "\r\n";
         if(cookies.length() > 0)
            request += "Cookie: " + cookies + "\r\n";
         request += "Upgrade: IoChat/0.1.0\r\n\r\n";
         mConnection.writeData(request.getBytes("ISO-8859-1"));

         String line = mConnection.read();
         if(!checkResponseLine(line))
            throw new java.io.IOException("Expected \"HTTP/1.1 101 Switching Protocols\", got \"" + line + "\".");

         while(!line.equals("") && !line.equals("\r"))
            line = mConnection.read();

         (new Thread(this)).start();
      }
      catch(java.io.IOException e)
      {
         chatError(e.getMessage());
         chatError("Failed to connect.");
         e.printStackTrace();
      }

      return (mWindow == null)?"true":"false";
   }

   public void submit(String text)
   {
      try {mConnection.write(text);} catch(java.io.IOException e) {}
   }

   public void beep()
   {
      if(mBeepClip != null)
         mBeepClip.play();
   }

   public void run()
   {
      try
      {
         while(true)
         {
            String line = mConnection.read();
            if(line == null)
               throw new java.io.IOException("Connection terminated.");
            chatCommand(new Command(line));
         }
      }
      catch(Throwable e)
      {
         chatError(e.getMessage());
         chatError("Lost connection to server.");
         e.printStackTrace();
      }
   }

   private void chatCommand(Command command)
   {
      if(mWindow == null)
      {
         synchronized(this) {mCommands.addElement(command);}
      }
      else
      {
         Object[] args = {command.name, command.parameters, (command.text != null)?"true":"false", command.text};
         mWindow.call("chatCommand", args);
      }
   }

   private void chatError(String message)
   {
      chatCommand(new Command("error", message));
   }

   public Object[][] pollCommands()
   {
      Object[][] result;

      synchronized(this)
      {
         int count = mCommands.size();
         result = new Object[count][];

         for(int i = 0; i < count; i++)
         {
            Command command = (Command)mCommands.elementAt(i);
            result[i] = new Object[4];
            result[i][0] = command.name;
            result[i][1] = command.parameters;
            result[i][2] = (command.text != null)?"true":"false";
            result[i][3] = command.text;
         }

         mCommands.removeAllElements();
      }

      return result;
   }

   private boolean checkResponseLine(String line)
   {
      if(!line.startsWith("HTTP/1."))
         return false;

      int pos = line.indexOf(' ');
      if(pos < 0)
         return false;

      if(!line.startsWith("101 ", pos+1))
         return false;
      return true;
   }
}
